# encoding: utf-8
"""
@project: djangoModel->push_single
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 单人推送
@created_time: 2023/8/10 10:13
"""
from rest_framework.decorators import api_view
from rest_framework.views import APIView

from ..services.push_single_service import PushSingleService
from ..utils.custom_response import util_response
from ..utils.custom_tool import request_params_wrapper


class PushSingleApi(APIView):

    @api_view(["GET"])
    @request_params_wrapper
    def list(self, *args, request_params, **kwargs):
        filter_fields = request_params.get("filter_fields")
        only_first = request_params.get("only_first", False)
        need_pagination = request_params.get("need_pagination", True)
        data, err = PushSingleService.list(
            params=request_params,
            filter_fields=filter_fields,
            only_first=only_first,
            need_pagination=need_pagination
        )
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)

    @api_view(["POST"])
    @request_params_wrapper
    def add(self, *args, request_params, **kwargs):
        data, err = PushSingleService.add(params=request_params)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)

    @api_view(["PUT"])
    @request_params_wrapper
    def edit(self, *args, request_params, **kwargs):
        data, err = PushSingleService.edit(params=request_params)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)

    @api_view(["PUT"])
    @request_params_wrapper
    def batch_dit(self, *args, request_params, **kwargs):
        data, err = PushSingleService.batch_edit(
            params=request_params
        )
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)

    @api_view(["DELETE"])
    @request_params_wrapper
    def delete(self, *args, request_params, **kwargs):
        data, err = PushSingleService.delete(params=request_params)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)

    @api_view(["GET"])
    @request_params_wrapper
    def group_list(self, *args, request_params, **kwargs):
        data, err = PushSingleService.group_list(params=request_params)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)



from django.forms import model_to_dict

from ..models import *
from ..utils.custom_tool import format_params_handle, force_transform_type


class PushTemplateService:

    @staticmethod
    def add(params: dict = None, **kwargs):
        """
        推送添加
        :param params: 添加参数子字典
        :param kwargs:
        :return:
        """
        # 参数整合与空值验证
        params, is_void = force_transform_type(variable=params, var_type="dict", default={})
        kwargs, is_void = force_transform_type(variable=kwargs, var_type="dict", default={})
        params.update(kwargs)
        # 过滤主表修改字段
        try:
            main_form_data = format_params_handle(
                param_dict=params.copy(),
                is_remove_empty=True,
                filter_filed_list=[
                    "value",
                    "template",
                    "default_params",
                    "title",
                    "send_type",
                    "files",
                    "push_times",
                    "push_success",
                    "push_total",
                    "created_time"
                ],
                alias_dict={},
                is_validate_type=True
            )
        except ValueError as e:
            # 模型字段验证
            return None, str(e)
        # IO操作
        try:
            # 主表插入数据
            push = PushTemplate.objects.create(**main_form_data)
        except Exception as e:
            return None, f'''{str(e)} in "{str(e.__traceback__.tb_frame.f_globals["__file__"])}" : Line {str(
                e.__traceback__.tb_lineno)}'''

        return {"id": push.id}, None

    @staticmethod
    def edit(params: dict = None, push_id=None, search_param: dict = None, **kwargs):
        """
        推送编辑
        :param params: 修改的参数
        :param push_id: 需要修改的推送主键
        :param search_param: 搜索参数, 服务层根据信息等其他搜索字段检索到需要修改的数据
        :return: data, err
        """
        # 空值检查
        params, is_pass = force_transform_type(variable=params, var_type="dict", default={})
        push_id, is_pass = force_transform_type(variable=push_id, var_type="int")
        search_param, is_pass = force_transform_type(variable=search_param, var_type="dict", default={})
        if not push_id and not search_param:
            return None, "无法找到要修改数据，请检查参数"

        # 搜索字段过滤
        if search_param:
            search_param = format_params_handle(
                param_dict=search_param,
                filter_filed_list=[
                    "id|int"
                ],
                alias_dict={"id_list": "id__in", "[": "id__in"}
            )
        # 修改内容检查处理
        try:
            main_form_data = format_params_handle(
                param_dict=params,
                is_validate_type=True,
                is_remove_empty=True,
                filter_filed_list=[
                    "value",
                    "template",
                    "default_params",
                    "title",
                    "send_type",
                    "files",
                    "push_times",
                    "push_success",
                    "push_total",
                    "created_time"
                ],
            )
        except ValueError as e:
            return None, str(e)
        # 构建ORM，检查是否存在可修改项目
        push_obj = PushTemplate.objects
        if push_id:
            push_obj = push_obj.filter(id=push_id)
        elif search_param:
            push_obj = push_obj.filter(**search_param)

        update_total = push_obj.count()
        if update_total == 0:
            return None, "没有找到可修改项目"

        # IO 操作
        try:
            push_obj.update(**main_form_data)
        except Exception as e:
            return None, "修改异常:" + str(e)
        return push_obj.first().to_json(), None

    @staticmethod
    def list(params):
        page = int(params['page']) - 1 if 'page' in params else 0
        size = int(params['size']) if 'size' in params else 10
        push = PushTemplate.objects
        # invoice = invoice.order_by('-send_time')
        push = push.order_by('-id')

        params = format_params_handle(
            param_dict=params,
            is_remove_empty=True,
            filter_filed_list=[
                "id|int", "id_list|list",
                "created_time_start|date", "created_time_end|date"
            ],
            split_list=["id_list"],
            alias_dict={
                "created_time_start": "created_time__gte", "created_time_end": "created_time__lte", "id_list": "id__in"
            },
        )
        push = push.extra(select={'created_time': 'DATE_FORMAT(send_time, "%%Y-%%m-%%d %%H:%%i:%%s")'})
        push = push.filter(**params).values()
        total = push.count()
        #
        current_page_set = push[page * size: page * size + size] if page >= 0 and size > 0 else push
        res_list = []
        for i, it in enumerate(current_page_set):
            it['order'] = page * size + i + 1
            it['created_time'] = it['created_time'].strftime("%Y-%m-%d %H:%M:%S") if it['created_time'] else it[
                'created_time']

            res_list.append(it)

        data = res_list

        return {'size': int(size), 'page': int(page + 1), 'total': total, 'list': data, }, None

    @staticmethod
    def detail(push_id):
        if not push_id:
            return None, "推送id不能为空"
        push = PushTemplate.objects.filter(id=push_id)
        push = push.extra(select={'created_time': 'DATE_FORMAT(created_time, "%%Y-%%m-%%d %%H:%%i:%%s")'})

        push = push.first()
        if not push:
            return None, "无法找到要查询的数据，请检查参数"
        data = model_to_dict(push)
        return data, None



import json
import sys
import time
import os
import string
import random
import pytz
import datetime
from django.utils import timezone
from dateutil.parser import parse
from dateutil.tz import tzlocal
import pandas as pd
from datetime import datetime
from decimal import Decimal


# 获取当前时间
def get_current_time():
    # TODO USE_TZ = False 时会报错 如果USE_TZ设置为True时，Django会使用系统默认设置的时区，即America/Chicago，此时的TIME_ZONE不管有没有设置都不起作用。
    tz = pytz.timezone('Asia/Shanghai')
    # 返回datetime格式的时间
    now_time = timezone.now().astimezone(tz=tz).strftime("%Y-%m-%d %H:%M:%S.%f")
    # now = datetime.strptime(now_time, '%Y-%m-%d %H:%M:%S')
    now = datetime.now().strptime(now_time, "%Y-%m-%d %H:%M:%S.%f")
    return now


# 时间精确到微秒
def append_microsecond_to_datetime(datetime_str: str) -> datetime:
    """
    将当前微秒追加到日期时间字符串中。如果字符串仅包含日期，当前时间也附加在微秒之前。
    """
    # Check if the datetime string contains a time
    if len(datetime_str.split(' ')) == 2:
        current_microsecond = datetime.now().strftime("%f")
        datetime_str = f'{datetime_str}.{current_microsecond}'
        dt = datetime.strptime(datetime_str, "%Y-%m-%d %H:%M:%S.%f")
    else:
        # Get the current time and microsecond
        current_time_microsecond = datetime.now().strftime("%H:%M:%S.%f")
        datetime_str = f'{datetime_str} {current_time_microsecond}'
        dt = datetime.strptime(datetime_str, "%Y-%m-%d %H:%M:%S.%f")
    return dt


# 数据key替换
def replace_dict_key(dictionary, old_key, new_key):
    if old_key in dictionary:
        dictionary[new_key] = dictionary.pop(old_key)
    return dictionary


# 数据字典key替换
def replace_key_in_dict(original_dict, old_key, new_key):
    copied_dict = original_dict.copy()
    if old_key in copied_dict:
        copied_dict[new_key] = copied_dict.pop(old_key)
    return copied_dict


# 数据列表key替换
def replace_key_in_list_dicts(list_dicts, old_keys, new_key):
    for d in list_dicts:
        for old_key in old_keys:
            if old_key in d:
                d[new_key] = d.pop(old_key)
    return list_dicts


# 数据列表字典key替换
def replace_key_in_list_replacement_dicts(list_dicts, replacement_dict):
    for d in list_dicts:
        for old_key, new_key in replacement_dict.items():
            if old_key in d:
                d[new_key] = d.pop(old_key)
    return list_dicts


# 字符串转列表
def parse_integers(value):
    if isinstance(value, str):
        if "," in value:
            lst = [int(num) for num in value.split(",")]
        else:
            lst = [int(value)]
    elif isinstance(value, int):
        lst = [value]
    else:
        raise TypeError("不支持的值类型。应为字符串或int")

    return lst


# 保留两位小数
def keep_two_decimal_places(str_num):
    result_num = format(float(str_num), "")

    if len(result_num.split(".")[-1]) < 2:
        result_num = result_num + "0"
    return result_num


# 生成一个长度为16的密码
def generate_password(length=16):
    # 合并所有可能的字符，包括大小写字母、数字和标点符号
    # all_chars = string.ascii_letters + string.digits + string.punctuation
    all_chars = string.ascii_letters + string.digits
    # length = random.randint(8, 12)
    # 随机选择指定数量的字符
    password = ''.join(random.choice(all_chars) for _ in range(length))

    return password


# 数字表示生成几位, True表示生成带有字母的 False不带字母的
def get_code(n=6, alpha=False):
    s = ''  # 创建字符串变量,存储生成的验证码
    for i in range(n):  # 通过for循环控制验证码位数
        num = random.randint(1, 9)  # 生成随机数字0-9
        if alpha:  # 需要字母验证码,不用传参,如果不需要字母的,关键字alpha=False
            upper_alpha = chr(random.randint(65, 90))
            lower_alpha = chr(random.randint(97, 122))
            num = random.choice([num, upper_alpha, lower_alpha])
        s = s + str(num)
    return s


# 检查列表字段是否存在
def find(list, keyword):
    try:
        list.index(keyword)
        return True
    except ValueError:
        return False


# 批量数据时间格式化
def format_dates(data, date_fields):
    for item in data:
        for field in date_fields:
            if field in item and item[field]:
                try:
                    # 如果字段已经是 datetime 对象，就无需解析
                    if isinstance(item[field], datetime):
                        date = item[field]
                    else:
                        # 尝试解析并格式化日期
                        date = parse(item[field])
                    # 使用 strftime 格式化日期
                    item[field] = date.astimezone(tzlocal()).strftime('%Y-%m-%d %H:%M:%S')
                except ValueError:
                    # 如果解析失败，保留原来的值
                    pass
    return data


# 列表根据指定key计算value求和
def aggregate_data(data_list, group_field, agg_fields):
    # 将数据转换为 DataFrame
    df = pd.DataFrame(data_list)

    # 将累加字段转换为数值型
    for field in agg_fields:
        df[field] = pd.to_numeric(df[field])

    # 根据 group_field 对数据进行分组，并对 agg_fields 进行求和
    grouped = df.groupby(group_field)[agg_fields].sum().reset_index()

    # 将结果转换回字典列表
    grouped_data = grouped.to_dict('records')

    return grouped_data


# json模板替换
def replace_placeholders(data, replacements):
    # 将 data 转换为 JSON 格式的字符串
    data_str = str(data)

    # 依次替换每一个 "{{}}"
    for replacement in replacements:
        data_str = data_str.replace("{{}}", replacement, 1)

    # 将字符串重新转换为字典
    data = eval(data_str)

    return data


# 获取程序运行时间
def testRunTime():
    start = datetime.now()
    for i in range(1000):
        for j in range(500):
            m = i + j
            print(m)
    end = datetime.now()
    print(end - start)
    return end - start


# json转换
def convert_dict_to_json(data):
    def convert_values_to_str(value):
        if isinstance(value, dict):
            return {convert_values_to_str(key): convert_values_to_str(val) for key, val in value.items()}
        elif isinstance(value, list):
            return [convert_values_to_str(val) for val in value]
        elif isinstance(value, Decimal):
            return str(value)
        elif value is None:
            return value
        else:
            return str(value)

    converted_data = convert_values_to_str(data)
    json_str = json.dumps(converted_data, ensure_ascii=False)
    return json_str


# 占位符字典替换
def replace_values(dict1, dict2):
    if isinstance(dict2, str):
        dict2 = json.loads(dict2)

    if isinstance(dict2, dict):
        for key in dict2:
            if key == 'data' and isinstance(dict2[key], dict):
                for sub_key in dict2[key]:
                    if isinstance(dict2[key][sub_key], dict) and 'value' in dict2[key][sub_key] and dict2[key][sub_key]['value'] == '{{}}' and sub_key in dict1:
                        # Convert the number to a string before replacing
                        dict2[key][sub_key]['value'] = str(dict1[sub_key]) if isinstance(dict1[sub_key], int) else dict1[sub_key]
            elif isinstance(dict2[key], dict):
                replace_values(dict1, dict2[key])
    return dict2




# encoding: utf-8
"""
"""
from rest_framework.decorators import api_view
from rest_framework.views import APIView

from ..services.push_plan_service import PushPlanService
from ..utils.custom_response import util_response
from ..utils.custom_tool import request_params_wrapper, flow_service_wrapper
from ..utils.user_wrapper import user_authentication_force_wrapper, user_authentication_wrapper


class PushPlanApis(APIView):
    # 推送添加
    @api_view(['POST'])
    @request_params_wrapper
    def add(self, *args, user_info, request_params, **kwargs, ):
        params = request_params
        user_id = user_info.get("user_id")
        platform_id = user_info.get("platform_id")
        params.setdefault("user_id", user_id)  # 用户ID
        push_set, err = PushPlanService.add(params)
        if err is None:
            return util_response(data=push_set)
        return util_response(err=47767, msg=err)

    # 推送修改
    @api_view(['PUT'])
    @request_params_wrapper
    def edit(self, *args, user_info, request_params, **kwargs, ):
        params = request_params
        user_id = user_info.get("user_id")
        platform_id = user_info.get("platform_id")
        push_id = params.get("push_id", 0)
        push_set, err = PushPlanService.edit(params, push_id)
        if err is None:
            return util_response(data=push_set)
        return util_response(err=47767, msg=err)

    # 推送列表
    @api_view(['GET'])
    @request_params_wrapper
    def list(self, *args, user_info, request_params, **kwargs, ):
        params = request_params
        # ============   字段验证处理 start ============
        user_id = user_info.get("user_id")
        params.setdefault("user_id", user_id)  # 用户ID

        push_set, err = PushPlanService.list(params)

        if err is None:
            return util_response(data=push_set)

        return util_response(err=47767, msg=err)



# -*- coding: utf-8 -*-
import os
import configparser

from alipay import AliPay
from django.conf import settings
from django.contrib.sites import requests

from pathlib import Path

from config.config import Config


project_root = Config.absolutePath('')
module_root = str(Path(__file__).resolve().parent)
private_key = open(Config.absolutePath(Config.getIns().get("xj_payment", "APP_PRIVATE_KEY_FILE", "/config/app_private_key.pem"))).read()
public_key = open(Config.absolutePath(Config.getIns().get("xj_payment","ALIPAY_PUBLIC_KEY_FILE", "/config/alipay_public_key.pem"))).read()
app_id = Config.getIns().get("xj_payment","ALIPAY_APP_ID","456")



def my_ali_pay(notify_url=None):
    """
    支付宝支付对象
    :param notify_url:
    支付成功支付宝服务器异步通知默认回调url，会向这个地址发送POST请求，接口实现校验是否支付已经完成，注意：此地址需要能在公网进行访问
    :return: 支付对象
    """

    alipay = AliPay(
        appid=app_id,
        # appid=settings.ALIPAY_APP_ID,  # APPID
        app_notify_url=notify_url,  # 默认回调url,可以传也可以不传
        # app_notify_url=notify_url,  # 默认回调url,可以传也可以不传
        app_private_key_string=private_key,
        # app_private_key_string=settings.APP_PRIVATE_KEY,  # 应用私钥
        # alipay_public_key_string=settings.ALIPAY_PUBLIC_KEY,  # 支付宝公钥
        alipay_public_key_string=public_key,
        sign_type="RSA2",  # RSA 或者 RSA2
        # debug=settings.ALI_PAY_DEBUG  # 默认False，沙箱环境改成True
        debug=True
    )

    return alipay


def is_app_pay(order_string):
    """
    判断是否是App支付
    :param order_string: 签名后的订单信息
    :return: True or False
    支付宝支付功能对应的方法:
    注意: App支付不需要传支付网关ALI_PAY_URL
    电脑网站支付: alipay.trade.page.pay
    手机网站支付: alipay.trade.wap.pay
    App支付: alipay.trade.app.pay
    小程序支付: alipay.trade.create
    当面付(条码支付): alipay.trade.pay
    交易预创建(扫码支付): alipay.trade.precreate
    """
    order_dict = dict()
    for i in order_string.split('&'):
        temp_list = i.split("=")
        order_dict[temp_list[0]] = temp_list[1]

    method = order_dict.get("method")

    return True if "app" in method else False



# encoding: utf-8
"""
"""
from rest_framework.decorators import api_view
from rest_framework.views import APIView

from ..services.push_template_main_service import PushTemplateMainService
from ..utils.custom_response import util_response
from ..utils.custom_tool import request_params_wrapper


class PushApis(APIView):
    # 推送添加
    @api_view(['POST'])
    @request_params_wrapper
    def template_send(self, *args, request_params, **kwargs, ):
        params = request_params
        type = params.get("type")
        value = params.get("value")
        user_id = params.get("user_id")
        replacements = params.get("replacements")
        push_set, err = PushTemplateMainService.template_send(type, value, user_id, replacements)
        if err is None:
            return util_response(data=push_set)
        return util_response(err=47767, msg=err)



# encoding: utf-8
"""
@project: djangoModel->push_single_service
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 单挑推送
@created_time: 2023/8/10 10:33
"""
from django.core.paginator import Paginator, EmptyPage
from django.db.models import F

from xj_push.models import PushSingle
from ..orator_models.base_model import base_db
from ..utils.custom_tool import force_transform_type, format_params_handle, filter_fields_handler, write_to_log, dynamic_load_class
from ..utils.j_recur import JRecur


class PushSingleService:
    @staticmethod
    def add(params: dict = None, **kwargs):
        """
        添加推送记录
        :param params: 搜索参数
        :param kwargs: 最省参数
        :return: data, err
        """
        # -------------------------- section 强制类型，字段过滤--------------------------------
        kwargs, is_pass = force_transform_type(variable=kwargs, var_type="dict", default={})
        params, is_pass = force_transform_type(variable=params, var_type="dict", default={})
        params.update(kwargs)

        try:
            params = format_params_handle(
                param_dict=params, is_validate_type=True,
                filter_filed_list=[
                    "to_user_id|int", "source_code|str", "template_id|int", "template_params|only_dict", "thread_id|int",
                    "title|str", "content|str", "send_type|str", "files|json", "is_jump_link|int", "link|str"
                ],
            )
            must_keys = ["title", "content", "to_user_id"]
            for i in must_keys:
                if not params.get(i):
                    return None, str(i) + " 必填"
        except ValueError as e:
            return None, str(e)
        # -------------------------- section 强制类型，字段过滤 --------------------------------

        # -------------------------- section 构建ORM --------------------------------
        try:
            push_single_record = PushSingle.objects.create(**params)
        except Exception as e:
            write_to_log(prefix="PushSingleService。add:", content=e)
            return None, "插入错误"

        return {"push_single_record": push_single_record.id}, None
        # -------------------------- section 构建ORM --------------------------------

    @staticmethod
    def edit(params: dict = None, **kwargs):
        # -------------------------- section 强制类型，字段过滤--------------------------------
        kwargs, is_pass = force_transform_type(variable=kwargs, var_type="dict", default={})
        params, is_pass = force_transform_type(variable=params, var_type="dict", default={})
        params.update(kwargs)
        pk, is_pass = force_transform_type(variable=params.get("pk", params.get("id")), var_type="int")
        if not pk:
            return None, "ID错误"
        try:
            params = format_params_handle(
                param_dict=params, is_validate_type=True,
                is_remove_empty=True,
                filter_filed_list=[
                    "to_user_id|int", "source_code|str", "title|str", "content|str",
                    "send_type|str", "files|json", "is_read|int", "is_delete|int", "thread_id|int",
                    "template_id|int", "template_params|only_dict", "is_jump_link|int", "link|str"
                ]
            )
        except ValueError as e:
            return None, str(e)
        # -------------------------- section 强制类型，字段过滤 --------------------------------

        # -------------------------- section 构建ORM --------------------------------
        # 仅仅查询第一条
        try:
            push_single_record = PushSingle.objects.filter(id=pk)
            if not push_single_record.first():
                return None, "数据不存在，无法修改"
            push_single_record.update(**params)
        except Exception as e:
            write_to_log(prefix="PushSingleService。add:", content=e)
            return None, "插入错误"

        return None, None
        # -------------------------- section 构建ORM --------------------------------

    @staticmethod
    def batch_edit(params: dict = None, search_params: dict = None, **kwargs):
        # -------------------------- section 强制类型，字段过滤--------------------------------
        kwargs, is_pass = force_transform_type(variable=kwargs, var_type="dict", default={})
        params, is_pass = force_transform_type(variable=params, var_type="dict", default={})
        params.update(kwargs)

        if not search_params:
            search_params, err = JRecur.get_filter_tree_params(
                params=params,
                prefix="search_"
            )

        search_params = format_params_handle(
            param_dict=search_params or {},
            is_validate_type=True,
            is_remove_empty=True,
            filter_filed_list=[
                "to_user_id|int", "source_code|str", "title|str", "content|str", "thread_id|int",
                "send_type|str", "template_id|int", "is_jump_link|int"
            ],
            alias_dict={"title": "title__contains"}
        )
        if not search_params:
            return None, "搜索条件为空"

        try:
            params = format_params_handle(
                param_dict=params, is_validate_type=True,
                is_remove_empty=False,
                filter_filed_list=[
                    "to_user_id|int", "source_code|str", "title|str", "content|str",
                    "send_type|str", "files|json", "is_read|int", "is_delete|int", "thread_id|int",
                    "template_id|int", "template_params|only_dict", "is_jump_link|int", "link|str"
                ]
            )
        except ValueError as e:
            return None, str(e)
        # -------------------------- section 强制类型，字段过滤 --------------------------------

        # -------------------------- section 构建ORM --------------------------------
        try:
            push_single_record = PushSingle.objects.filter(**search_params)
            push_single_record.update(**params)
        except Exception as e:
            write_to_log(prefix="PushSingleService。add:", content=e)
            return None, "插入错误"

        return None, None
        # -------------------------- section 构建ORM --------------------------------

    @staticmethod
    def delete(params: dict = None, **kwargs):
        kwargs, is_pass = force_transform_type(variable=kwargs, var_type="dict", default={})
        params, is_pass = force_transform_type(variable=params, var_type="dict", default={})
        params.update(kwargs)
        pk, is_pass = force_transform_type(variable=params.get("pk") or params.get("id"), var_type="int")
        if not pk:
            return None, "ID不能为空"
        push_single_record = PushSingle.objects.filter(id=pk).first()
        if push_single_record:
            push_single_record.delete()
        return None, None

    @staticmethod
    def list(params: dict = None, filter_fields: "list|str" = None, only_first: bool = False, need_pagination: bool = True, **kwargs):
        """
        单挑推送记录
        :param params: 搜索参数
        :param filter_fields: 过滤字段
        :param only_first: 是否仅仅查询第一条
        :param need_pagination: 是否分页
        :param kwargs: 最省参数
        :return: data, err
        """
        # -------------------------- section 强制类型 --------------------------------
        kwargs, is_pass = force_transform_type(variable=kwargs, var_type="dict", default={})
        params, is_pass = force_transform_type(variable=params, var_type="dict", default={})
        params.update(kwargs)
        size, is_pass = force_transform_type(variable=params.pop('size', 10), var_type="int", default=10)
        page, is_pass = force_transform_type(variable=params.pop('page', 1), var_type="int", default=1)
        # -------------------------- section 强制类型 --------------------------------

        # -------------------------- section 参数过滤 --------------------------------
        # 允许查询字段过滤
        params = format_params_handle(
            param_dict=params,
            filter_filed_list=[
                "id|int", "to_user_id|int", "source_code|str", "title|str", "content|str",
                "send_type|str", "files|json", "is_read|int", "is_delete|int", "thread_id|int",
                "created_time_start|date", "created_time_end|date", "is_jump_link|int"
            ],
            alias_dict={"created_time_start": "created_time__gte", "created_time_end": "created_time__lte"}
        )
        # 处理filter_fields，获取ORM查询字段列表
        filter_fields_list = filter_fields_handler(
            input_field_expression=filter_fields,
            all_field_list=[
                "id", "to_user_id", "source_code", "title", "content", "template_id", "template_params", "template_template",
                "send_type", "files", "is_read", "is_delete", "created_time", "is_jump_link", "thread_id", "link"
            ]
        )
        # -------------------------- section 参数过滤 --------------------------------

        # -------------------------- section 构建ORM --------------------------------
        push_single_record = PushSingle.objects.extra(
            select={'created_time': 'DATE_FORMAT(push_single.created_time, "%%Y-%%m-%%d %%H:%%i:%%s")'}
        ).annotate(
            template_template=F("template__template")
        ).filter(**params).values(*filter_fields_list)

        # 仅仅查询第一条
        if only_first:
            push_single_record = push_single_record.first()
            push_single_record["template_msg"] = push_single_record["template_template"]
            for k, v in push_single_record.get("template_params").items():
                push_single_record["template_msg"] = push_single_record["template_msg"].replace("{" + k + "}", v)
            return push_single_record, None

        # 不分页查询
        total = push_single_record.count()
        if not need_pagination and total <= 200:
            finish_list = list(push_single_record)
            return finish_list, None

        # 分页查询
        paginator = Paginator(push_single_record, size)
        try:
            enroll_obj = paginator.page(page)
        except EmptyPage:
            return {'total': total, "page": page, "size": size, 'list': []}, None
        except Exception as e:
            return None, f'{str(e)}'
        finish_list = list(enroll_obj.object_list)

        # 替换模板变量
        for i in finish_list:
            i["template_msg"] = i["template_template"]
            if not i.get("template_template") or not i.get("template_params"):
                continue
            for k, v in i.get("template_params").items():
                i["template_msg"] = i["template_msg"].replace("{" + k + "}", v)

        # 信息列表获取
        thread_id_list = [i["thread_id"] for i in push_single_record if i.get("thread_id")]
        ThreadListService, import_err = dynamic_load_class(
            import_path="xj_thread.services.thread_list_service",
            class_name="ThreadListService"
        )
        thread_map = {}
        if not import_err:
            thread_map, err = ThreadListService.search(id_list=thread_id_list, need_map=True)
        for i in finish_list:
            i["thread_info"] = thread_map.get(i["thread_id"] or 0)

        # 报名列表拼接
        EnrollServices, import_err = dynamic_load_class(
            import_path="xj_enroll.service.enroll_services",
            class_name="EnrollServices"
        )
        enroll_map = {}
        if not import_err:
            enroll_list, err = EnrollServices.enroll_list(params={"thread_id_list": thread_id_list}, need_pagination=False)
            enroll_map = {str(i["thread_id"]): i for i in enroll_list}
        for i in finish_list:
            i["enroll_info"] = enroll_map.get(str(i["thread_id"]) or "0")

        return {'total': total, "page": page, "size": size, 'list': finish_list}, None
        # -------------------------- section 构建ORM --------------------------------

    @staticmethod
    def group_list(params: dict = None, filter_fields: "list|str" = None, only_first: bool = False, need_pagination: bool = True, **kwargs):
        """
        单挑推送分组查询记录
        :param params: 搜索参数
        :param filter_fields: 过滤字段
        :param only_first: 是否仅仅查询第一条
        :param need_pagination: 是否分页
        :param kwargs: 最省参数
        :return: data, err
        """
        kwargs, is_pass = force_transform_type(variable=kwargs, var_type="dict", default={})
        params, is_pass = force_transform_type(variable=params, var_type="dict", default={})
        params.update(kwargs)

        # 参数处理
        size, is_pass = force_transform_type(variable=params.pop('size', 10), var_type="int", default=10)
        page, is_pass = force_transform_type(variable=params.pop('page', 1), var_type="int", default=1)
        source_code, is_pass = force_transform_type(variable=params.get("source_code"), var_type="str")
        to_user_id, is_pass = force_transform_type(variable=params.get("to_user_id"), var_type="int")

        # 构建ORM
        query_set = base_db.table("push_single").select_raw("""
            to_user_id,
            source_code,
            title,
            content,
            template_id,
            template_params,
            send_type,
            files,
            is_read,
            is_delete,
            count(id) as total,
            date_format(created_time, "%%Y-%%m-%%d %%H:%%m:%%s") as created_time
        """).group_by("source_code").order_by("created_time", "desc")

        total_query = base_db.table("push_single").select_raw("count( DISTINCT source_code ) as total")
        un_read_total_query = base_db.table("push_single").select_raw("count(id) as un_read_total")
        un_read_total_map_query = base_db.table("push_single").select_raw("""
                    source_code, count(id) as un_read_total
                """).where("is_read", 0)

        if source_code:
            query_set = query_set.where("source_code", "=", source_code)
            total_query = total_query.where("source_code", "=", source_code)

        if to_user_id:
            query_set = query_set.where("to_user_id", "=", to_user_id)
            total_query = total_query.where("to_user_id", "=", to_user_id)
            un_read_total_map_query = un_read_total_map_query.where("to_user_id", "=", to_user_id)
            un_read_total_query = un_read_total_query.where("to_user_id", "=", to_user_id)

        # 汇总查询
        total = total_query.pluck("total")
        if total == 0:
            return {"page": page, "size": size, "total": total, "un_read_total": 0, "list": []}, None

        # 分页查询
        query_set = query_set.paginate(size, page)
        un_read_total = un_read_total_query.where("is_read", 0).pluck("un_read_total")
        un_read_total_map = un_read_total_map_query.where_in("source_code", [i["source_code"] for i in query_set]).group_by("source_code").lists("un_read_total", "source_code")

        for i in query_set:
            i["un_read_total"] = un_read_total_map.get(i["source_code"], 0)

        return {"page": page, "size": size, "total": total, "un_read_total": un_read_total, "list": list(query_set)}, None



# encoding: utf-8
"""
@project: djangoModel->pipline_base
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 管道基础类
@created_time: 2023/7/20 11:09
"""
from abc import ABC, abstractmethod
import time

from config.config import JConfig
from ..utils.custom_tool import force_transform_type, dynamic_load_class, write_to_log

config = JConfig()


class PipelineBase(ABC):
    def __init__(self, *args, **kwargs):
        pass

    @staticmethod
    def _setdefault(params: dict = None, update_params: dict = None):
        """
        合并字典，防止覆盖原有的值
        """
        # 强制类型转换
        params, err = force_transform_type(variable=params, var_type="only_dict", default={})
        update_params, err = force_transform_type(variable=update_params, var_type="only_dict", default={})

        # 开始合并
        for k, v in update_params.items():
            params.setdefault(k, v)
        return params, None

    @staticmethod
    def _get_user_info(user_id: int = None, prefix=""):
        """
        获取用户信息
        :param user_id: 用户ID
        :return: 用户信息,err
        """
        user_info = {}
        try:
            app_id = config.get(section="xj_user", option="app_id", default="")

            user_id, err = force_transform_type(variable=user_id, var_type="int")
            if err:
                return user_info, err

            # 获取用户详细信息
            DetailInfoService, err = dynamic_load_class(import_path="xj_user.services.user_detail_info_service", class_name="DetailInfoService")
            if not err:
                user_detail, err = DetailInfoService.get_detail(user_id=user_id)
                user_detail, err = force_transform_type(variable=user_detail, var_type="only_dict", default={})
                if not err:
                    user_info.update(user_detail)

            # 获取用户的单点登录信息
            # UserSsoServeService, err = dynamic_load_class(import_path="xj_user.services.user_sso_serve_service", class_name="UserSsoServeService")
            # if not err:
            #     user_sso_info, err = UserSsoServeService.user_sso_to_user(user_id=user_id, app_id=app_id)
            #     user_sso_info, err = force_transform_type(variable=user_sso_info, var_type="only_dict", default={})
            #     if not err:
            #         user_info.update(user_sso_info)
        except Exception as e:
            write_to_log(prefix="推送管道异常", err_obj=e)
        return {prefix + k: v for k, v in user_info.items()}, None

    @staticmethod
    def _get_enroll_info(enroll_id: int = None, prefix=""):
        """
        获取报名信息，项目信息
        :param enroll_id: 报名ID
        :return: 报名信息，获取失败的错误信息
        """
        enroll_info = {}
        try:
            enroll_id, err = force_transform_type(variable=enroll_id, var_type="int")
            thread_id = None
            if err:
                return enroll_info, err

            # 获取信息标数据
            EnrollServices, err = dynamic_load_class(import_path="xj_enroll.service.enroll_services", class_name="EnrollServices")
            if not err:
                enroll_detail, err = EnrollServices.enroll_detail(enroll_id=enroll_id, simple_return=True)
                enroll_detail, err = force_transform_type(variable=enroll_detail, var_type="only_dict", default={})
                enroll_info.update(enroll_detail)
                thread_id = enroll_info.get("thread_id", None)

            # 获取项目信息
            ThreadItemService, err = dynamic_load_class(import_path="xj_thread.services.thread_item_service", class_name="ThreadItemService")
            if thread_id and not err:
                thread_info, err = ThreadItemService.detail(pk=thread_id)
                thread_info, err = force_transform_type(variable=thread_info, var_type="only_dict", default={})
                enroll_info.update(thread_info)
        except Exception as e:
            write_to_log(prefix="推送管道异常", err_obj=e)
        return {prefix + k: v for k, v in enroll_info.items()}, None

    @staticmethod
    def __get_invoice_info(invoice_id: int = None, prefix=""):
        """
        开票的获取服务
        :param invoice_id:
        """
        invoice_info = {}
        try:
            invoice_id, err = force_transform_type(variable=invoice_id, var_type="int")
            if err:
                return invoice_id, err

            InvoiceService, err = dynamic_load_class(import_path="xj_invoice.services.invoice_service", class_name="InvoiceService")
            if not err:
                invoice_detail_info, err = InvoiceService.detail(invoice_id)
                invoice_detail_info, err = force_transform_type(variable=invoice_detail_info, var_type="only_dict", default={})
                invoice_info.update(invoice_detail_info)
        except Exception as e:
            write_to_log(prefix="推送管道异常", err_obj=e)
        return {prefix + k: v for k, v in invoice_info.items()}, None, None

    @staticmethod
    def _get_finance_info(finance_id: int = None, order_no=None, transact_no=None, prefix=""):
        finance_info = {}
        # 强制类型转换
        try:
            finance_id, err = force_transform_type(variable=finance_id, var_type="int")
            order_no, err = force_transform_type(variable=order_no, var_type="str")
            transact_no, err = force_transform_type(variable=transact_no, var_type="str")
            if not finance_id and not transact_no and not order_no:
                return finance_info, err

            # 自动动态引入
            FinanceListService, err = dynamic_load_class(import_path="xj_finance.services.finance_list_service", class_name="FinanceListService")
            if not err:
                transact_detail, err = FinanceListService.detail(
                    pk=finance_id,
                    order_no=order_no,
                    transact_no=transact_no
                )
                transact_detail, err = force_transform_type(variable=transact_detail, var_type="only_dict", default={})
                finance_info.update(transact_detail)
        except Exception as e:
            write_to_log(prefix="推送管道异常", err_obj=e)
        return {prefix + k: v for k, v in finance_info.items()}, None, None

    def init_params(self, *args, params: dict = None, **kwargs):
        params, err = force_transform_type(variable=params, var_type="only_dict", default={})
        kwargs, err = force_transform_type(variable=kwargs, var_type="only_dict", default={})
        params.update(kwargs)
        params["push_current_time"] = time.strftime("%Y-%m-%d %H:%M:%S")

    @abstractmethod
    def process(*args, params: dict = None, **kwargs):
        # 当前的推送时间
        pass



# encoding: utf-8
"""
@project: djangoModel->single_push_pipline
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 单挑推送管道
@created_time: 2023/8/23 13:13
"""
from django.db.models import F

from xj_enroll.models import EnrollSubitemRecord
from xj_push.push_pipeline.pipline_base import PipelineBase
from xj_push.services.push_single_service import PushSingleService
from ..utils.custom_tool import force_transform_type, dynamic_load_class, write_to_log


class SinglePushPipeline(PipelineBase):
    notice_title = {
        "publish": "提交订单",
        "enroll": "项目报名",
        "appoint": "选择镖师",
        "pay": "完成支付",
        "upload": "上传标书文件",
        "accept_in_checking": "审核通过标书资料",
        "check_fial_upload": "操作驳回",
        "accept": "操作验收",
        "cancel_publish": "取消订单",
    }

    @staticmethod
    def process(*args, params: dict = None, **kwargs):
        # 获取报名相关的信息
        params, err = force_transform_type(variable=params, var_type="only_dict", default={})
        kwargs, err = force_transform_type(variable=kwargs, var_type="only_dict", default={})
        params.update(kwargs)
        enroll_id = params.get("enroll_id", None)
        if not enroll_id:
            return None, "报名ID错误"

        # 导入依赖
        Enroll, enroll_import_err = dynamic_load_class(import_path="xj_enroll.models", class_name="Enroll")
        EnrollRecord, record_import_err = dynamic_load_class(import_path="xj_enroll.models", class_name="EnrollRecord")
        if enroll_import_err or record_import_err:
            write_to_log(prefix="站内推送管道异常", content="请安装报名模块")
            return None, None

        print("params >>>>>>>>>>", params, "\n")
        # 获取相关信息
        enroll_info = Enroll.objects.filter(id=enroll_id).values().first()
        thread_id = enroll_info.get("thread_id")
        enroll_user_id = enroll_info.get("user_id")
        add_params = {"thread_id": thread_id, "source_code": "订单动态"}

        # ------------------------- section 管道挂油，管道分流 start ------------------------
        # 发布的订单后通知客户
        if params.get("push_action") == "publish":
            add_params["content"] = add_params["title"] = "订单发布成功"
            add_params["to_user_id"] = enroll_user_id
            data, err = PushSingleService.add(params=add_params)

        # 镖师报名相互通知
        elif params.get("push_action") == "enroll":
            # 提醒客户有人报名
            add_params["content"] = add_params["title"] = "项目报名"
            add_params["to_user_id"] = enroll_user_id
            data, err = PushSingleService.add(params=add_params)
            # 体校镖师报名成功
            record_info = EnrollRecord.objects.filter(id=params.get("record_id")).values().first()
            if record_info:
                add_params["content"] = add_params["title"] = "报名成功"
                add_params["to_user_id"] = record_info.get("user_id")
                data, err = PushSingleService.add(params=add_params)

        # 用户取消订单通知所有的报名用户
        elif params.get("push_action") == "cancel_publish":
            # 通知所有报名的用户
            add_params["content"] = add_params["title"] = "客户取消订单"
            record_infos = list(EnrollRecord.objects.filter(enroll_id=enroll_id).values())
            for i in record_infos:
                if not i.get("user_id"):
                    continue
                add_params["to_user_id"] = i.get("user_id")
                PushSingleService.add(params=add_params)

            # 通知客户取消成功
            add_params["content"] = add_params["title"] = "取消订单成功"
            add_params["to_user_id"] = enroll_user_id
            PushSingleService.add(params=add_params)

        # 指派通知
        elif params.get("push_action") == "appoint":
            # 通知客户指派成功
            add_params["content"] = add_params["title"] = "选择镖师成功"
            add_params["to_user_id"] = enroll_user_id
            data, err = PushSingleService.add(params=add_params)

            # 通知镖师您已被指派
            record_info = EnrollRecord.objects.filter(id=params.get("record_id")).first().values()
            if record_info:
                add_params["content"] = add_params["title"] = "您已被指派"
                add_params["to_user_id"] = record_info.get("user_id")
                data, err = PushSingleService.add(params=add_params)

            # 没有指派的镖师给予提示
            un_appoint_records = list(EnrollRecord.objects.exclude(id=params.get("record_id")).values())
            for i in un_appoint_records:
                add_params["content"] = add_params["title"] = "很遗憾，您没有选中，再接再厉"
                add_params["to_user_id"] = i.get("user_id")
                data, err = PushSingleService.add(params=add_params)

        # 已支付通知客户
        elif params.get("push_action") == "payed":
            add_params["content"] = add_params["title"] = "支付成功"
            add_params["to_user_id"] = enroll_user_id
            PushSingleService.add(params=add_params)

        # 镖师上传
        elif params.get("push_action") == "upload":
            print("执行")
            add_params["content"] = add_params["title"] = "镖书文件已上传"
            # 通知客户
            add_params["to_user_id"] = enroll_user_id
            PushSingleService.add(params=add_params)

            # 通知镖师
            subitem_record = EnrollSubitemRecord.objects.annotate(
                record_user_id=F("enroll_record__user_id")
            ).filter(id=params.get("id")).values("record_user_id", "id").first()
            if subitem_record:
                add_params["to_user_id"] = subitem_record.get("record_user_id")
                PushSingleService.add(params=add_params)

        # 标书初审通过
        elif params.get("push_action") == "check_success":
            add_params["content"] = add_params["title"] = "镖书文件初审成功"
            # 通知客户
            add_params["to_user_id"] = enroll_user_id
            PushSingleService.add(params=add_params)

            # 通知镖师
            bx_worker = EnrollRecord.objects.filter(enroll_id=enroll_id).exclude(enroll_status_code=124).values().first()
            if bx_worker:
                add_params["to_user_id"] = bx_worker.get("user_id")
                PushSingleService.add(params=add_params)

        # 标书初审失败
        elif params.get("push_action") == "check_fail":
            add_params["content"] = add_params["title"] = "镖书文件初审失败"
            # 通知客户
            add_params["to_user_id"] = enroll_user_id
            PushSingleService.add(params=add_params)

            # 通知镖师
            bx_worker = EnrollRecord.objects.filter(enroll_id=enroll_id).exclude(enroll_status_code=124).values().first()
            if bx_worker:
                add_params["to_user_id"] = bx_worker.get("user_id")
                PushSingleService.add(params=add_params)

        # 标书初审通过
        elif params.get("push_action") == "accept_success":
            add_params["content"] = add_params["title"] = "镖书文件验收成功"
            # 通知客户
            add_params["to_user_id"] = enroll_user_id
            PushSingleService.add(params=add_params)

            # 通知镖师
            bx_worker = EnrollRecord.objects.filter(enroll_id=enroll_id).exclude(enroll_status_code=124).values().first()
            if bx_worker:
                add_params["to_user_id"] = bx_worker.get("user_id")
                PushSingleService.add(params=add_params)

        elif params.get("push_action") == "accept_fail":  # 标书初审失败
            add_params["content"] = add_params["title"] = "镖书文件验收失败"
            # 通知客户
            add_params["to_user_id"] = enroll_user_id
            PushSingleService.add(params=add_params)

            # 通知镖师
            bx_worker = EnrollRecord.objects.filter(enroll_id=enroll_id).exclude(enroll_status_code=124).values().first()
            if bx_worker:
                add_params["to_user_id"] = bx_worker.get("user_id")
                PushSingleService.add(params=add_params)

        # ------------------------- section 管道挂油 end  ------------------------
        return None, None
